@@ -31,7 +31,7 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define BAYGUI_VERSION   "BayGUI build "__DATE__
 
 #define GUISERVER_NAME   "BAYGUI.EX5"
-#define LAUNCHER_NAME    "/APPS/GLAUNCH.EX5"
+#define LAUNCHER_NAME    "/APPS/BAYGUI/GLAUNCH.EX5"
 #define WALLPAPER_NAME   "/SERVERS/MONAWALL.BM5"
 #define KEYSERVER_NAME   "KEYBDMNG.EX2"
 #define MOUSESERVER_NAME "MOUSE.EX2"
